@@ -13,6 +13,8 @@
 #include <type_traits>
 #include <vector>
 
+#include "xtl/xclosure.hpp"
+
 #include "xutils.hpp"
 
 namespace xt
@@ -39,7 +41,6 @@ namespace xt
     template <class D>
     class xexpression
     {
-
     public:
 
         using derived_type = D;
@@ -90,7 +91,7 @@ namespace xt
      * Returns a constant reference to the actual derived type of the xexpression.
      */
     template <class D>
-    inline auto xexpression<D>::derived_cast() && noexcept->derived_type
+    inline auto xexpression<D>::derived_cast() && noexcept -> derived_type
     {
         return *static_cast<derived_type*>(this);
     }
@@ -99,7 +100,7 @@ namespace xt
     namespace detail
     {
         template <class E>
-        struct is_xexpression_impl : std::is_base_of<xexpression<E>, E>
+        struct is_xexpression_impl : std::is_base_of<xexpression<std::decay_t<E>>, std::decay_t<E>>
         {
         };
 
@@ -128,13 +129,13 @@ namespace xt
     template <class E, class EN = void>
     struct xclosure
     {
-        using type = closure_t<E>;
+        using type = xtl::closure_type_t<E>;
     };
 
     template <class E>
     struct xclosure<E, disable_xexpression<std::decay_t<E>>>
     {
-        using type = xscalar<closure_t<E>>;
+        using type = xscalar<xtl::closure_type_t<E>>;
     };
 
     template <class E>
@@ -143,13 +144,13 @@ namespace xt
     template <class E, class EN = void>
     struct const_xclosure
     {
-        using type = const_closure_t<E>;
+        using type = xtl::const_closure_type_t<E>;
     };
 
     template <class E>
     struct const_xclosure<E, disable_xexpression<std::decay_t<E>>>
     {
-        using type = xscalar<const_closure_t<E>>;
+        using type = xscalar<xtl::const_closure_type_t<E>>;
     };
 
     template <class E>
@@ -196,7 +197,9 @@ namespace xt
         inline typename E::reference get_element(E& e, S i, Args... args)
         {
             if (sizeof...(Args) >= e.dimension())
+            {
                 return get_element(e, args...);
+            }
             return e(i, args...);
         }
 
@@ -210,10 +213,69 @@ namespace xt
         inline typename E::const_reference get_element(const E& e, S i, Args... args)
         {
             if (sizeof...(Args) >= e.dimension())
+            {
                 return get_element(e, args...);
+            }
             return e(i, args...);
         }
     }
+
+    /*************************
+     * expression tag system *
+     *************************/
+
+    struct xtensor_expression_tag {};
+
+    namespace detail
+    {
+        template <class E, class = void_t<int>>
+        struct get_expression_tag
+        {
+            using type = xtensor_expression_tag;
+        };
+
+        template <class E>
+        struct get_expression_tag<E, void_t<typename std::decay_t<E>::expression_tag>>
+        {
+            using type = typename std::decay_t<E>::expression_tag;
+        };
+
+        template <class E>
+        using get_expression_tag_t = typename get_expression_tag<E>::type;
+
+        template <class... T>
+        struct expression_tag_and;
+
+        template <class T>
+        struct expression_tag_and<T>
+        {
+            using type = T;
+        };
+
+        template <class T>
+        struct expression_tag_and<T, T>
+        {
+            using type = T;
+        };
+
+        template <class T1, class... T>
+        struct expression_tag_and<T1, T...>
+            : expression_tag_and<T1, typename expression_tag_and<T...>::type>
+        {
+        };
+
+        template <class... T>
+        using expression_tag_and_t = typename expression_tag_and<T...>::type;
+    }
+
+    template <class... T>
+    struct xexpression_tag
+    {
+        using type = detail::expression_tag_and_t<detail::get_expression_tag_t<T>...>;
+    };
+
+    template <class... T>
+    using xexpression_tag_t = typename xexpression_tag<T...>::type;
 }
 
 #endif
