@@ -93,19 +93,6 @@ namespace xt
         size_type m_size;
     };
 
-    /**
-     * Returns a slice representing an interval, to
-     * be used as an argument of view function.
-     * @param min_val the first index of the interval
-     * @param max_val the last index of the interval
-     * @sa view
-     */
-    template <class T, class E = std::enable_if_t<!std::is_same<T, placeholders::xtuph>::value>>
-    inline auto range(T min_val, T max_val) noexcept
-    {
-        return xrange<T>(min_val, max_val);
-    }
-
     /******************************
      * xstepped_range declaration *
      ******************************/
@@ -132,20 +119,6 @@ namespace xt
         size_type m_step;
     };
 
-    /**
-     * Returns a slice representing an interval, to
-     * be used as an argument of view function.
-     * @param min_val the first index of the interval
-     * @param max_val the last index of the interval
-     * @param step the space between two indices
-     * @sa view
-     */
-    template <class T, class E = std::enable_if_t<!std::is_same<T, placeholders::xtuph>::value>>
-    inline auto range(T min_val, T max_val, T step) noexcept
-    {
-        return xstepped_range<T>(min_val, max_val, step);
-    }
-
     /********************
      * xall declaration *
      ********************/
@@ -217,6 +190,10 @@ namespace xt
         return xnewaxis_tag();
     }
 
+    /******************
+     * xrange_adaptor *
+     ******************/
+
     template <class A, class B, class C>
     struct xrange_adaptor
     {
@@ -226,21 +203,45 @@ namespace xt
         }
 
         template <class MI = A, class MA = B, class STEP = C>
-        inline std::enable_if_t<!std::is_integral<MI>::value && std::is_integral<MA>::value && std::is_integral<STEP>::value, xstepped_range<int>>
+        inline std::enable_if_t<std::is_integral<MI>::value &&
+                                std::is_integral<MA>::value &&
+                                std::is_integral<STEP>::value, xstepped_range<int>>
+        get(std::size_t size) const
+        {
+            return xstepped_range<int>(m_min, m_max, m_step);
+        }
+
+        template <class MI = A, class MA = B, class STEP = C>
+        inline std::enable_if_t<!std::is_integral<MI>::value && 
+                                std::is_integral<MA>::value &&
+                                std::is_integral<STEP>::value, xstepped_range<int>>
         get(std::size_t size) const
         {
             return xstepped_range<int>(m_step > 0 ? 0 : int(size) - 1, m_max, m_step);
         }
 
         template <class MI = A, class MA = B, class STEP = C>
-        inline std::enable_if_t<std::is_integral<MI>::value && !std::is_integral<MA>::value && std::is_integral<STEP>::value, xstepped_range<int>>
+        inline std::enable_if_t<std::is_integral<MI>::value &&
+                                !std::is_integral<MA>::value &&
+                                std::is_integral<STEP>::value, xstepped_range<int>>
         get(std::size_t size) const
         {
             return xstepped_range<int>(m_min, m_step > 0 ? int(size) : -1, m_step);
         }
 
         template <class MI = A, class MA = B, class STEP = C>
-        inline std::enable_if_t<!std::is_integral<MI>::value && !std::is_integral<MA>::value && std::is_integral<STEP>::value, xstepped_range<int>>
+        inline std::enable_if_t<std::is_integral<MI>::value &&
+                                std::is_integral<MA>::value &&
+                               !std::is_integral<STEP>::value, xrange<int>>
+        get(std::size_t size) const
+        {
+            return xrange<int>(m_min, m_max);
+        }
+
+        template <class MI = A, class MA = B, class STEP = C>
+        inline std::enable_if_t<!std::is_integral<MI>::value &&
+                                !std::is_integral<MA>::value &&
+                                std::is_integral<STEP>::value, xstepped_range<int>>
         get(std::size_t size) const
         {
             int min_val_arg = m_step > 0 ? 0 : int(size) - 1;
@@ -249,21 +250,27 @@ namespace xt
         }
 
         template <class MI = A, class MA = B, class STEP = C>
-        inline std::enable_if_t<std::is_integral<MI>::value && !std::is_integral<MA>::value && !std::is_integral<STEP>::value, xrange<std::size_t>>
+        inline std::enable_if_t<std::is_integral<MI>::value &&
+                                !std::is_integral<MA>::value &&
+                                !std::is_integral<STEP>::value, xrange<std::size_t>>
         get(std::size_t size) const
         {
             return xrange<std::size_t>((std::size_t)m_min, size);
         }
 
         template <class MI = A, class MA = B, class STEP = C>
-        inline std::enable_if_t<!std::is_integral<MI>::value && std::is_integral<MA>::value && !std::is_integral<STEP>::value, xrange<std::size_t>>
+        inline std::enable_if_t<!std::is_integral<MI>::value &&
+                                std::is_integral<MA>::value &&
+                                !std::is_integral<STEP>::value, xrange<std::size_t>>
         get(std::size_t /*size*/) const
         {
             return xrange<std::size_t>(0, (std::size_t)m_max);
         }
 
         template <class MI = A, class MA = B, class STEP = C>
-        inline std::enable_if_t<!std::is_integral<MI>::value && !std::is_integral<MA>::value && !std::is_integral<STEP>::value, xall<std::size_t>>
+        inline std::enable_if_t<!std::is_integral<MI>::value &&
+                                !std::is_integral<MA>::value &&
+                                !std::is_integral<STEP>::value, xall<std::size_t>>
         get(std::size_t size) const
         {
             return xall<std::size_t>(size);
@@ -297,13 +304,13 @@ namespace xt
     inline disable_xslice<S, std::size_t> get_size(const S&) noexcept
     {
         return 1;
-    };
+    }
 
     template <class S>
     inline auto get_size(const xslice<S>& slice) noexcept
     {
         return slice.derived_cast().size();
-    };
+    }
 
     /*******************************************************
      * homogeneous step_size for integral types and slices *
@@ -328,13 +335,14 @@ namespace xt
     template <class S, class I>
     inline disable_xslice<S, std::size_t> value(const S& s, I) noexcept
     {
-        return s;
+        return static_cast<std::size_t>(s);
     }
 
     template <class S, class I>
     inline auto value(const xslice<S>& slice, I i) noexcept
     {
-        return slice.derived_cast()(i);
+        using ST = typename S::size_type;
+        return slice.derived_cast()(static_cast<ST>(i));
     }
 
     /****************************************
